# 2019-02-26 David Goncalves <david@lestat.st>
#            Python class to communicate with an AM2315 chip on I2C bus.
#            It's an AOSONG temperature and humidity sensor.
#            > Code ported to micropython

import sys
import time
import ustruct


class uam2315:

    def __init__(self, bus, address=0x5c):
        self.i2c_bus = bus
        self.i2c_addr = address

    def _wakeup_sensor(self):
        # This command will fail but is just intended to wakeup the device
        try:
            self.i2c_bus.writeto(self.i2c_addr, b'')

        except:
            pass

    def _verify_crc(self, data):
        crc = 0xFFFF
        for c in data:
            crc = crc ^ c
            for i in range(1, 9):
                if (crc & 0x01):
                    crc = crc >> 1
                    crc = crc ^ 0xA001

                else:
                    crc = crc >> 1

        return ustruct.pack('BB', crc & 0x00FF, crc >> 8)

    def _send_data(self, data):
        self._wakeup_sensor()

        self.i2c_bus.writeto_mem(self.i2c_addr, data[0], bytearray(data[1:]))

        if (data[0] == 0x03):
            read_len = data[2] + 4

        elif (data[0] == 0x10):
            read_len = 4

        else:
            raise(Exception("Unsupported chip command '%h'" % data[0]))

        resp = self.i2c_bus.readfrom_mem(self.i2c_addr, 0, read_len)
        crc = self._verify_crc(resp[:-2])

        if (crc != resp[-2:]):
            raise(Exception("CRC mismatch... error during transmission"))

        return(resp)

    def GetTemperature(self):
        data = self._send_data([0x03, 0x02, 0x02])
        temp = (((data[2] & 0x7F) << 8) + data[3]) / 10.0

        if (data[2] >> 7) == 1:
            temp = - temp

        return(temp)

    def GetHumidity(self):
        data = self._send_data([0x03, 0x00, 0x02])
        humidity = ((data[2] << 8) + data[3]) / 10.0

        return(humidity)
